/*
 * main.c
 * This file is part of Network-inador
 *
 * Copyright (C) 2024 - Félix Arreola Rodríguez
 *
 * Network-inador is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Network-inador is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Network-inador; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, 
 * Boston, MA  02110-1301  USA
 */

#ifndef __NETWORK_INADOR_H__
#define __NETWORK_INADOR_H__

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>

#include <sys/types.h>
#include <unistd.h>

#include <linux/if_addr.h>

#ifndef FALSE
#define FALSE 0
#endif

#ifndef TRUE
#define TRUE !FALSE
#endif

/* Función que define una lectura desde un FD */
typedef void (*NetworkInadorInputFunc) (void *, int, int);
typedef void (*NetworkInadorChildWatchFunc) (void *, pid_t, int);

struct NetworkInadorOps {
	/* Vigilar FDs */
	unsigned int (*input_add) (int fd, int cond, NetworkInadorInputFunc func, void *user_data);
	void (*input_remove) (unsigned int);
	
	/* Vigilar procesos */
	unsigned int (*process_watch) (pid_t pid, NetworkInadorChildWatchFunc, void *user_data);
	void (*process_unwatch) (unsigned int);
	
	/* Ver si agregamos timers aquí */
};

typedef struct _NetworkInadorHandle NetworkInadorHandle;
typedef struct _Interface Interface;
typedef struct _IPAddr IPAddr;

NetworkInadorHandle * network_inador_init_handle (struct NetworkInadorOps *network_inador_ops);
void network_inador_destroy_handle (NetworkInadorHandle *handle);

void network_inador_setup_child_handler (NetworkInadorHandle *handle);

/* Funciones básicas de interfaces */
int interfaces_change_mac_address (NetworkInadorHandle *handle, int index, void *new_mac);
int interfaces_change_mtu (NetworkInadorHandle *handle, int index, uint32_t new_mtu);
int interfaces_change_set_up (NetworkInadorHandle *handle, int index);
int interfaces_change_set_down (NetworkInadorHandle *handle, int index);
int interfaces_change_name (NetworkInadorHandle *handle, int index, char * new_name);

/* Funciones básicas de dhcp */
int interfaces_dhcp_client_run (NetworkInadorHandle *handle, int index, int type, uint32_t flags);
int interfaces_dhcp_client_stop (NetworkInadorHandle *handle, int index);

/* Lista de getters */
uint32_t *network_inador_list_ifaces (NetworkInadorHandle *handle);
Interface * network_inador_get_iface (NetworkInadorHandle *handle, uint32_t index);
const unsigned char *network_inador_iface_get_name (Interface *iface);
uint32_t network_inador_iface_get_index (Interface *iface);
uint32_t network_inador_iface_get_link_type (Interface *iface);
uint32_t network_inador_iface_get_master_index (Interface *iface);
uint32_t network_inador_iface_get_mtu (Interface *iface);
uint16_t network_inador_iface_get_flags (Interface *iface);
uint8_t network_inador_iface_get_is_wireless (Interface *iface);
uint8_t network_inador_iface_get_num_ips (Interface *iface);
IPAddr *network_inador_iface_get_ipaddr_by_index (Interface *iface, int index);
Interface *network_inador_ipaddr_get_iface (IPAddr *ip_addr);
uint8_t network_inador_ipaddr_get_family (IPAddr *ip_addr);
uint8_t network_inador_ipaddr_get_prefix (IPAddr *ip_addr);
uint8_t network_inador_ipaddr_has_local (IPAddr *ip_addr);
uint8_t network_inador_ipaddr_has_brd (IPAddr *ip_addr);
uint8_t network_inador_ipaddr_get_scope (IPAddr *ip_addr);
uint32_t network_inador_ipaddr_get_flags (IPAddr *ip_addr);
void network_inador_ipaddr_get_cacheinfo (IPAddr *ip_addr, struct ifa_cacheinfo *cacheinfo);
void network_inador_ipaddr_get_addr (IPAddr *ip_addr, void *addr, int *addr_size);
void network_inador_ipaddr_get_local_addr (IPAddr *ip_addr, void *addr, int *addr_size);
void network_inador_ipaddr_get_brd_addr (IPAddr *ip_addr, void *addr, int *addr_size);
const unsigned char *network_inador_ipaddr_get_label (IPAddr *ip_addr);

#endif /* __NETWORK_INADOR_H__ */

